@@ -35,15 +35,19 @@ func (Postpone) Execute(aerc *widgets.Aerc, args []string) error {
 	if acct == nil {
 		return errors.New("No account selected")
 	}
-	composer, _ := aerc.SelectedTab().(*widgets.Composer)
+	tab := aerc.SelectedTab()
+	if tab == nil {
+		return errors.New("No tab selected")
+	}
+	composer, _ := tab.Content.(*widgets.Composer)
 	config := composer.Config()
-	tabName := aerc.TabNames()[aerc.SelectedTabIndex()]
+	tabName := tab.Name
 
 	if config.Postpone == "" {
 		return errors.New("No Postpone location configured")
 	}
 
-	aerc.Logger().Println("Postponing mail")
+	logging.Infof("Postponing mail")
 
 	header, err := composer.PrepareHeader()
 	if err != nil {
@@ -76,7 +80,7 @@ func (Postpone) Execute(aerc *widgets.Aerc, args []string) error {
 
 		handleErr := func(err error) {
 			aerc.PushError(err.Error())
-			aerc.Logger().Println("Postponing failed:", err)
+			logging.Errorf("Postponing failed: %v", err)
 			aerc.NewTab(composer, tabName)
 		}
 
