@@ -3,7 +3,6 @@ package config
 import (
 	"errors"
 	"fmt"
-	"io/ioutil"
 	"log"
 	"net/url"
 	"os"
@@ -18,44 +17,64 @@ import (
 
 	"github.com/gdamore/tcell/v2"
 	"github.com/go-ini/ini"
+	"github.com/google/shlex"
 	"github.com/imdario/mergo"
 	"github.com/kyoh86/xdg"
+	"github.com/mitchellh/go-homedir"
 
 	"git.sr.ht/~rjarry/aerc/lib/templates"
+	"git.sr.ht/~rjarry/aerc/logging"
 )
 
 type GeneralConfig struct {
-	DefaultSavePath string `ini:"default-save-path"`
+	DefaultSavePath    string `ini:"default-save-path"`
+	PgpProvider        string `ini:"pgp-provider"`
+	UnsafeAccountsConf bool   `ini:"unsafe-accounts-conf"`
 }
 
 type UIConfig struct {
-	IndexFormat         string        `ini:"index-format"`
-	TimestampFormat     string        `ini:"timestamp-format"`
-	ThisDayTimeFormat   string        `ini:"this-day-time-format"`
-	ThisWeekTimeFormat  string        `ini:"this-week-time-format"`
-	ThisYearTimeFormat  string        `ini:"this-year-time-format"`
-	ShowHeaders         []string      `delim:","`
-	RenderAccountTabs   string        `ini:"render-account-tabs"`
-	PinnedTabMarker     string        `ini:"pinned-tab-marker"`
-	SidebarWidth        int           `ini:"sidebar-width"`
-	PreviewHeight       int           `ini:"preview-height"`
-	EmptyMessage        string        `ini:"empty-message"`
-	EmptyDirlist        string        `ini:"empty-dirlist"`
-	MouseEnabled        bool          `ini:"mouse-enabled"`
-	ThreadingEnabled    bool          `ini:"threading-enabled"`
-	FuzzyFolderComplete bool          `ini:"fuzzy-folder-complete"`
-	NewMessageBell      bool          `ini:"new-message-bell"`
-	Spinner             string        `ini:"spinner"`
-	SpinnerDelimiter    string        `ini:"spinner-delimiter"`
-	DirListFormat       string        `ini:"dirlist-format"`
-	DirListDelay        time.Duration `ini:"dirlist-delay"`
-	Sort                []string      `delim:" "`
-	NextMessageOnDelete bool          `ini:"next-message-on-delete"`
-	CompletionDelay     time.Duration `ini:"completion-delay"`
-	CompletionPopovers  bool          `ini:"completion-popovers"`
-	StyleSetDirs        []string      `ini:"stylesets-dirs" delim:":"`
-	StyleSetName        string        `ini:"styleset-name"`
-	style               StyleSet
+	AutoMarkRead                  bool          `ini:"auto-mark-read"`
+	IndexFormat                   string        `ini:"index-format"`
+	TimestampFormat               string        `ini:"timestamp-format"`
+	ThisDayTimeFormat             string        `ini:"this-day-time-format"`
+	ThisWeekTimeFormat            string        `ini:"this-week-time-format"`
+	ThisYearTimeFormat            string        `ini:"this-year-time-format"`
+	MessageViewTimestampFormat    string        `ini:"message-view-timestamp-format"`
+	MessageViewThisDayTimeFormat  string        `ini:"message-view-this-day-time-format"`
+	MessageViewThisWeekTimeFormat string        `ini:"message-view-this-week-time-format"`
+	MessageViewThisYearTimeFormat string        `ini:"message-view-this-year-time-format"`
+	ShowHeaders                   []string      `delim:","`
+	RenderAccountTabs             string        `ini:"render-account-tabs"`
+	PinnedTabMarker               string        `ini:"pinned-tab-marker"`
+	SidebarWidth                  int           `ini:"sidebar-width"`
+	PreviewHeight                 int           `ini:"preview-height"`
+	EmptyMessage                  string        `ini:"empty-message"`
+	EmptyDirlist                  string        `ini:"empty-dirlist"`
+	MouseEnabled                  bool          `ini:"mouse-enabled"`
+	ThreadingEnabled              bool          `ini:"threading-enabled"`
+	ForceClientThreads            bool          `ini:"force-client-threads"`
+	ClientThreadsDelay            time.Duration `ini:"client-threads-delay"`
+	FuzzyComplete                 bool          `ini:"fuzzy-complete"`
+	NewMessageBell                bool          `ini:"new-message-bell"`
+	Spinner                       string        `ini:"spinner"`
+	SpinnerDelimiter              string        `ini:"spinner-delimiter"`
+	IconUnencrypted               string        `ini:"icon-unencrypted"`
+	IconEncrypted                 string        `ini:"icon-encrypted"`
+	IconSigned                    string        `ini:"icon-signed"`
+	IconSignedEncrypted           string        `ini:"icon-signed-encrypted"`
+	IconUnknown                   string        `ini:"icon-unknown"`
+	IconInvalid                   string        `ini:"icon-invalid"`
+	DirListFormat                 string        `ini:"dirlist-format"`
+	DirListDelay                  time.Duration `ini:"dirlist-delay"`
+	DirListTree                   bool          `ini:"dirlist-tree"`
+	DirListCollapse               int           `ini:"dirlist-collapse"`
+	Sort                          []string      `delim:" "`
+	NextMessageOnDelete           bool          `ini:"next-message-on-delete"`
+	CompletionDelay               time.Duration `ini:"completion-delay"`
+	CompletionPopovers            bool          `ini:"completion-popovers"`
+	StyleSetDirs                  []string      `ini:"stylesets-dirs" delim:":"`
+	StyleSetName                  string        `ini:"styleset-name"`
+	style                         StyleSet
 	// customize border appearance
 	BorderCharVertical   rune `ini:"-"`
 	BorderCharHorizontal rune `ini:"-"`
@@ -68,6 +87,7 @@ const (
 	UI_CONTEXT_ACCOUNT
 	UI_CONTEXT_SUBJECT
 	BIND_CONTEXT_ACCOUNT
+	BIND_CONTEXT_FOLDER
 )
 
 type UIConfigContext struct {
@@ -81,6 +101,56 @@ const (
 	FILTER_HEADER
 )
 
+type RemoteConfig struct {
+	Value       string
+	PasswordCmd string
+	CacheCmd    bool
+	cache       string
+}
+
+func (c *RemoteConfig) parseValue() (*url.URL, error) {
+	return url.Parse(c.Value)
+}
+
+func (c *RemoteConfig) ConnectionString() (string, error) {
+	if c.Value == "" || c.PasswordCmd == "" {
+		return c.Value, nil
+	}
+
+	u, err := c.parseValue()
+	if err != nil {
+		return "", err
+	}
+
+	// ignore the command if a password is specified
+	if _, exists := u.User.Password(); exists {
+		return c.Value, nil
+	}
+
+	// don't attempt to parse the command if the url is a path (ie /usr/bin/sendmail)
+	if !u.IsAbs() {
+		return c.Value, nil
+	}
+
+	pw := c.cache
+
+	if pw == "" {
+		cmd := exec.Command("sh", "-c", c.PasswordCmd)
+		cmd.Stdin = os.Stdin
+		output, err := cmd.Output()
+		if err != nil {
+			return "", fmt.Errorf("failed to read password: %w", err)
+		}
+		pw = strings.TrimSpace(string(output))
+	}
+	u.User = url.UserPassword(u.User.Username(), pw)
+	if c.CacheCmd {
+		c.cache = pw
+	}
+
+	return u.String(), nil
+}
+
 type AccountConfig struct {
 	Archive           string
 	CopyTo            string
@@ -90,27 +160,43 @@ type AccountConfig struct {
 	Aliases           string
 	Name              string
 	Source            string
-	SourceCredCmd     string
 	Folders           []string
 	FoldersExclude    []string
 	Params            map[string]string
-	Outgoing          string
-	OutgoingCredCmd   string
+	Outgoing          RemoteConfig
 	SignatureFile     string
 	SignatureCmd      string
 	EnableFoldersSort bool     `ini:"enable-folders-sort"`
 	FoldersSort       []string `ini:"folders-sort" delim:","`
+	AddressBookCmd    string   `ini:"address-book-cmd"`
+	SendAsUTC         bool     `ini:"send-as-utc"`
+
+	// CheckMail
+	CheckMail        time.Duration `ini:"check-mail"`
+	CheckMailCmd     string        `ini:"check-mail-cmd"`
+	CheckMailTimeout time.Duration `ini:"check-mail-timeout"`
+	CheckMailInclude []string      `ini:"check-mail-include"`
+	CheckMailExclude []string      `ini:"check-mail-exclude"`
+
+	// PGP Config
+	PgpKeyId                string `ini:"pgp-key-id"`
+	PgpAutoSign             bool   `ini:"pgp-auto-sign"`
+	PgpOpportunisticEncrypt bool   `ini:"pgp-opportunistic-encrypt"`
+
+	// AuthRes
+	TrustedAuthRes []string `ini:"trusted-authres" delim:","`
 }
 
 type BindingConfig struct {
-	Global        *KeyBindings
-	AccountWizard *KeyBindings
-	Compose       *KeyBindings
-	ComposeEditor *KeyBindings
-	ComposeReview *KeyBindings
-	MessageList   *KeyBindings
-	MessageView   *KeyBindings
-	Terminal      *KeyBindings
+	Global                 *KeyBindings
+	AccountWizard          *KeyBindings
+	Compose                *KeyBindings
+	ComposeEditor          *KeyBindings
+	ComposeReview          *KeyBindings
+	MessageList            *KeyBindings
+	MessageView            *KeyBindings
+	MessageViewPassthrough *KeyBindings
+	Terminal               *KeyBindings
 }
 
 type BindingConfigContext struct {
@@ -121,10 +207,11 @@ type BindingConfigContext struct {
 }
 
 type ComposeConfig struct {
-	Editor         string     `ini:"editor"`
-	HeaderLayout   [][]string `ini:"-"`
-	AddressBookCmd string     `ini:"address-book-cmd"`
-	ReplyToSelf    bool       `ini:"reply-to-self"`
+	Editor              string         `ini:"editor"`
+	HeaderLayout        [][]string     `ini:"-"`
+	AddressBookCmd      string         `ini:"address-book-cmd"`
+	ReplyToSelf         bool           `ini:"reply-to-self"`
+	NoAttachmentWarning *regexp.Regexp `ini:"-"`
 }
 
 type FilterConfig struct {
@@ -140,7 +227,15 @@ type ViewerConfig struct {
 	Alternatives   []string
 	ShowHeaders    bool       `ini:"show-headers"`
 	AlwaysShowMime bool       `ini:"always-show-mime"`
+	ParseHttpLinks bool       `ini:"parse-http-links"`
 	HeaderLayout   [][]string `ini:"-"`
+	KeyPassthrough bool       `ini:"-"`
+}
+
+type StatuslineConfig struct {
+	RenderFormat string `ini:"render-format"`
+	Separator    string
+	DisplayMode  string `ini:"display-mode"`
 }
 
 type TriggersConfig struct {
@@ -149,7 +244,7 @@ type TriggersConfig struct {
 }
 
 type TemplateConfig struct {
-	TemplateDirs []string `ini:"template-dirs", delim:":"`
+	TemplateDirs []string `ini:"template-dirs" delim:":"`
 	NewMessage   string   `ini:"new-message"`
 	QuotedReply  string   `ini:"quoted-reply"`
 	Forwards     string   `ini:"forwards"`
@@ -159,15 +254,17 @@ type AercConfig struct {
 	Bindings        BindingConfig
 	ContextualBinds []BindingConfigContext
 	Compose         ComposeConfig
-	Ini             *ini.File       `ini:"-"`
-	Accounts        []AccountConfig `ini:"-"`
-	Filters         []FilterConfig  `ini:"-"`
-	Viewer          ViewerConfig    `ini:"-"`
-	Triggers        TriggersConfig  `ini:"-"`
+	Ini             *ini.File        `ini:"-"`
+	Accounts        []AccountConfig  `ini:"-"`
+	Filters         []FilterConfig   `ini:"-"`
+	Viewer          ViewerConfig     `ini:"-"`
+	Statusline      StatuslineConfig `ini:"-"`
+	Triggers        TriggersConfig   `ini:"-"`
 	Ui              UIConfig
 	ContextualUis   []UIConfigContext
 	General         GeneralConfig
 	Templates       TemplateConfig
+	Openers         map[string][]string
 }
 
 // Input: TimestampFormat
@@ -185,7 +282,7 @@ func mapName(raw string) string {
 	return string(newstr)
 }
 
-func loadAccountConfig(path string) ([]AccountConfig, error) {
+func loadAccountConfig(path string, accts []string) ([]AccountConfig, error) {
 	file, err := ini.Load(path)
 	if err != nil {
 		// No config triggers account configuration wizard
@@ -198,7 +295,11 @@ func loadAccountConfig(path string) ([]AccountConfig, error) {
 		if _sec == "DEFAULT" {
 			continue
 		}
+		if len(accts) > 0 && !contains(accts, _sec) {
+			continue
+		}
 		sec := file.Section(_sec)
+		sourceRemoteConfig := RemoteConfig{}
 		account := AccountConfig{
 			Archive:           "Archive",
 			Default:           "INBOX",
@@ -206,37 +307,58 @@ func loadAccountConfig(path string) ([]AccountConfig, error) {
 			Name:              _sec,
 			Params:            make(map[string]string),
 			EnableFoldersSort: true,
+			CheckMailTimeout:  10 * time.Second,
 		}
 		if err = sec.MapTo(&account); err != nil {
 			return nil, err
 		}
 		for key, val := range sec.KeysHash() {
-			if key == "folders" {
+			switch key {
+			case "folders":
 				folders := strings.Split(val, ",")
 				sort.Strings(folders)
 				account.Folders = folders
-			} else if key == "folders-exclude" {
+			case "folders-exclude":
 				folders := strings.Split(val, ",")
 				sort.Strings(folders)
 				account.FoldersExclude = folders
-			} else if key == "source-cred-cmd" {
-				account.SourceCredCmd = val
-			} else if key == "outgoing" {
-				account.Outgoing = val
-			} else if key == "outgoing-cred-cmd" {
-				account.OutgoingCredCmd = val
-			} else if key == "from" {
+			case "source":
+				sourceRemoteConfig.Value = val
+			case "source-cred-cmd":
+				sourceRemoteConfig.PasswordCmd = val
+			case "outgoing":
+				account.Outgoing.Value = val
+			case "outgoing-cred-cmd":
+				account.Outgoing.PasswordCmd = val
+			case "outgoing-cred-cmd-cache":
+				cache, err := strconv.ParseBool(val)
+				if err != nil {
+					return nil, fmt.Errorf(
+						"%s=%s %w", key, val, err)
+				}
+				account.Outgoing.CacheCmd = cache
+			case "from":
 				account.From = val
-			} else if key == "aliases" {
+			case "aliases":
 				account.Aliases = val
-			} else if key == "copy-to" {
+			case "copy-to":
 				account.CopyTo = val
-			} else if key == "archive" {
+			case "archive":
 				account.Archive = val
-			} else if key == "enable-folders-sort" {
+			case "enable-folders-sort":
 				account.EnableFoldersSort, _ = strconv.ParseBool(val)
-			} else if key != "name" {
-				account.Params[key] = val
+			case "pgp-key-id":
+				account.PgpKeyId = val
+			case "pgp-auto-sign":
+				account.PgpAutoSign, _ = strconv.ParseBool(val)
+			case "pgp-opportunistic-encrypt":
+				account.PgpOpportunisticEncrypt, _ = strconv.ParseBool(val)
+			case "address-book-cmd":
+				account.AddressBookCmd = val
+			default:
+				if key != "name" {
+					account.Params[key] = val
+				}
 			}
 		}
 		if account.Source == "" {
@@ -246,74 +368,82 @@ func loadAccountConfig(path string) ([]AccountConfig, error) {
 			return nil, fmt.Errorf("Expected from for account %s", _sec)
 		}
 
-		source, err := parseCredential(account.Source, account.SourceCredCmd)
+		source, err := sourceRemoteConfig.ConnectionString()
 		if err != nil {
-			return nil, fmt.Errorf("Invalid source credentials for %s: %s", _sec, err)
+			return nil, fmt.Errorf("Invalid source credentials for %s: %w", _sec, err)
 		}
 		account.Source = source
 
-		outgoing, err := parseCredential(account.Outgoing, account.OutgoingCredCmd)
+		_, err = account.Outgoing.parseValue()
 		if err != nil {
-			return nil, fmt.Errorf("Invalid outgoing credentials for %s: %s", _sec, err)
+			return nil, fmt.Errorf("Invalid outgoing credentials for %s: %w", _sec, err)
 		}
-		account.Outgoing = outgoing
 
 		accounts = append(accounts, account)
 	}
+	if len(accts) > 0 {
+		// Sort accounts struct to match the specified order, if we
+		// have one
+		if len(accounts) != len(accts) {
+			return nil, errors.New("account(s) not found")
+		}
+		sort.Slice(accounts, func(i, j int) bool {
+			return strings.ToLower(accts[i]) < strings.ToLower(accts[j])
+		})
+	}
 	return accounts, nil
 }
 
-func parseCredential(cred, command string) (string, error) {
-	if cred == "" || command == "" {
-		return cred, nil
+// Set at build time
+var shareDir string
+
+func buildDefaultDirs() []string {
+	var defaultDirs []string
+
+	prefixes := []string{
+		xdg.ConfigHome(),
+		xdg.DataHome(),
 	}
 
-	u, err := url.Parse(cred)
-	if err != nil {
-		return "", err
+	// Add XDG_CONFIG_HOME and XDG_DATA_HOME
+	for _, v := range prefixes {
+		if v != "" {
+			v, err := homedir.Expand(v)
+			if err != nil {
+				log.Println(err)
+			}
+			defaultDirs = append(defaultDirs, path.Join(v, "aerc"))
+		}
 	}
 
-	// ignore the command if a password is specified
-	if _, exists := u.User.Password(); exists {
-		return cred, nil
+	// Add custom buildtime shareDir
+	if shareDir != "" && shareDir != "/usr/local/share/aerc" {
+		shareDir, err := homedir.Expand(shareDir)
+		if err == nil {
+			defaultDirs = append(defaultDirs, shareDir)
+		}
 	}
 
-	// don't attempt to parse the command if the url is a path (ie /usr/bin/sendmail)
-	if !u.IsAbs() {
-		return cred, nil
-	}
+	// Add fixed fallback locations
+	defaultDirs = append(defaultDirs, "/usr/local/share/aerc")
+	defaultDirs = append(defaultDirs, "/usr/share/aerc")
 
-	cmd := exec.Command("sh", "-c", command)
-	cmd.Stdin = os.Stdin
-	output, err := cmd.Output()
-	if err != nil {
-		return "", fmt.Errorf("failed to read password: %s", err)
-	}
-
-	pw := strings.TrimSpace(string(output))
-	u.User = url.UserPassword(u.User.Username(), pw)
-
-	return u.String(), nil
+	return defaultDirs
 }
 
-var defaultDirs []string = []string{
-	path.Join(xdg.ConfigHome(), "aerc"),
-	path.Join(xdg.DataHome(), "aerc"),
-	"/usr/local/share/aerc",
-	"/usr/share/aerc",
-}
+var SearchDirs = buildDefaultDirs()
 
 func installTemplate(root, name string) error {
 	var err error
 	if _, err = os.Stat(root); os.IsNotExist(err) {
-		err = os.MkdirAll(root, 0755)
+		err = os.MkdirAll(root, 0o755)
 		if err != nil {
 			return err
 		}
 	}
 	var data []byte
-	for _, dir := range defaultDirs {
-		data, err = ioutil.ReadFile(path.Join(dir, name))
+	for _, dir := range SearchDirs {
+		data, err = os.ReadFile(path.Join(dir, name))
 		if err == nil {
 			break
 		}
@@ -321,7 +451,7 @@ func installTemplate(root, name string) error {
 	if err != nil {
 		return err
 	}
-	err = ioutil.WriteFile(path.Join(root, name), data, 0644)
+	err = os.WriteFile(path.Join(root, name), data, 0o644)
 	if err != nil {
 		return err
 	}
@@ -337,30 +467,41 @@ func (config *AercConfig) LoadConfig(file *ini.File) error {
 				Command: cmd,
 				Filter:  match,
 			}
-			if strings.Contains(match, ",~") {
+			switch {
+			case strings.Contains(match, ",~"):
 				filter.FilterType = FILTER_HEADER
-				header := filter.Filter[:strings.Index(filter.Filter, ",")]
+				header := filter.Filter[:strings.Index(filter.Filter, ",")] //nolint:gocritic // guarded by strings.Contains
 				regex := filter.Filter[strings.Index(filter.Filter, "~")+1:]
 				filter.Header = strings.ToLower(header)
 				filter.Regex, err = regexp.Compile(regex)
 				if err != nil {
 					return err
 				}
-			} else if strings.ContainsRune(match, ',') {
+			case strings.ContainsRune(match, ','):
 				filter.FilterType = FILTER_HEADER
-				header := filter.Filter[:strings.Index(filter.Filter, ",")]
+				header := filter.Filter[:strings.Index(filter.Filter, ",")] //nolint:gocritic // guarded by strings.Contains
 				value := filter.Filter[strings.Index(filter.Filter, ",")+1:]
 				filter.Header = strings.ToLower(header)
 				filter.Regex, err = regexp.Compile(regexp.QuoteMeta(value))
 				if err != nil {
 					return err
 				}
-			} else {
+			default:
 				filter.FilterType = FILTER_MIMETYPE
 			}
 			config.Filters = append(config.Filters, filter)
 		}
 	}
+	if openers, err := file.GetSection("openers"); err == nil {
+		for mimeType, command := range openers.KeysHash() {
+			mimeType = strings.ToLower(mimeType)
+			if args, err := shlex.Split(command); err != nil {
+				return err
+			} else {
+				config.Openers[mimeType] = args
+			}
+		}
+	}
 	if viewer, err := file.GetSection("viewer"); err == nil {
 		if err := viewer.MapTo(&config.Viewer); err != nil {
 			return err
@@ -374,23 +515,36 @@ func (config *AercConfig) LoadConfig(file *ini.File) error {
 			}
 		}
 	}
+	if statusline, err := file.GetSection("statusline"); err == nil {
+		if err := statusline.MapTo(&config.Statusline); err != nil {
+			return err
+		}
+	}
 	if compose, err := file.GetSection("compose"); err == nil {
 		if err := compose.MapTo(&config.Compose); err != nil {
 			return err
 		}
 		for key, val := range compose.KeysHash() {
-			switch key {
-			case "header-layout":
+			if key == "header-layout" {
 				config.Compose.HeaderLayout = parseLayout(val)
 			}
+
+			if key == "no-attachment-warning" && len(val) > 0 {
+				re, err := regexp.Compile("(?im)" + val)
+				if err != nil {
+					return fmt.Errorf(
+						"Invalid no-attachment-warning '%s': %w",
+						val, err,
+					)
+				}
+
+				config.Compose.NoAttachmentWarning = re
+			}
 		}
 	}
 
 	if ui, err := file.GetSection("ui"); err == nil {
-		if err := ui.MapTo(&config.Ui); err != nil {
-			return err
-		}
-		if err := validateBorderChars(ui, &config.Ui); err != nil {
+		if err := parseUiConfig(ui, &config.Ui); err != nil {
 			return err
 		}
 	}
@@ -405,33 +559,30 @@ func (config *AercConfig) LoadConfig(file *ini.File) error {
 			return err
 		}
 		uiSubConfig := UIConfig{}
-		if err := uiSection.MapTo(&uiSubConfig); err != nil {
+		if err := parseUiConfig(uiSection, &uiSubConfig); err != nil {
 			return err
 		}
-		if err := validateBorderChars(uiSection, &uiSubConfig); err != nil {
-			return err
+		contextualUi := UIConfigContext{
+			UiConfig: uiSubConfig,
 		}
-		contextualUi :=
-			UIConfigContext{
-				UiConfig: uiSubConfig,
-			}
 
 		var index int
-		if strings.Contains(sectionName, "~") {
+		switch {
+		case strings.Contains(sectionName, "~"):
 			index = strings.Index(sectionName, "~")
 			regex := string(sectionName[index+1:])
 			contextualUi.Regex, err = regexp.Compile(regex)
 			if err != nil {
 				return err
 			}
-		} else if strings.Contains(sectionName, "=") {
+		case strings.Contains(sectionName, "="):
 			index = strings.Index(sectionName, "=")
 			value := string(sectionName[index+1:])
 			contextualUi.Regex, err = regexp.Compile(regexp.QuoteMeta(value))
 			if err != nil {
 				return err
 			}
-		} else {
+		default:
 			return fmt.Errorf("Invalid Ui Context regex in %s", sectionName)
 		}
 
@@ -463,7 +614,7 @@ func (config *AercConfig) LoadConfig(file *ini.File) error {
 	}
 
 	// append default paths to template-dirs and styleset-dirs
-	for _, dir := range defaultDirs {
+	for _, dir := range SearchDirs {
 		config.Ui.StyleSetDirs = append(
 			config.Ui.StyleSetDirs, path.Join(dir, "stylesets"),
 		)
@@ -510,44 +661,92 @@ func (config *AercConfig) LoadConfig(file *ini.File) error {
 	return nil
 }
 
-func validateBorderChars(section *ini.Section, config *UIConfig) error {
+func parseUiConfig(section *ini.Section, config *UIConfig) error {
+	if err := section.MapTo(config); err != nil {
+		return err
+	}
+
+	if key, err := section.GetKey("border-char-vertical"); err == nil {
+		chars := []rune(key.String())
+		if len(chars) != 1 {
+			return fmt.Errorf("%v must be one and only one character", key)
+		}
+		config.BorderCharVertical = chars[0]
+	}
+	if key, err := section.GetKey("border-char-horizontal"); err == nil {
+		chars := []rune(key.String())
+		if len(chars) != 1 {
+			return fmt.Errorf("%v must be one and only one character", key)
+		}
+		config.BorderCharHorizontal = chars[0]
+	}
+
+	// Values with type=time.Duration must be explicitly set. If these
+	// values are given a default in the struct passed to ui.MapTo, which
+	// they are, a zero-value in the config won't overwrite the default.
+	if key, err := section.GetKey("dirlist-delay"); err == nil {
+		dur, err := key.Duration()
+		if err != nil {
+			return err
+		}
+		config.DirListDelay = dur
+	}
+	if key, err := section.GetKey("completion-delay"); err == nil {
+		dur, err := key.Duration()
+		if err != nil {
+			return err
+		}
+		config.CompletionDelay = dur
+	}
+
+	if config.MessageViewTimestampFormat == "" {
+		config.MessageViewTimestampFormat = config.TimestampFormat
+	}
+	if config.MessageViewThisDayTimeFormat == "" {
+		config.MessageViewThisDayTimeFormat = config.TimestampFormat
+	}
+	if config.MessageViewThisWeekTimeFormat == "" {
+		config.MessageViewThisWeekTimeFormat = config.TimestampFormat
+	}
+	if config.MessageViewThisDayTimeFormat == "" {
+		config.MessageViewThisDayTimeFormat = config.TimestampFormat
+	}
+
+	return nil
+}
+
+func validatePgpProvider(section *ini.Section) error {
+	m := map[string]bool{
+		"gpg":      true,
+		"internal": true,
+	}
 	for key, val := range section.KeysHash() {
-		switch key {
-		case "border-char-vertical":
-			char := []rune(val)
-			if len(char) != 1 {
-				return fmt.Errorf("%v must be one and only one character", key)
+		if key == "pgp-provider" {
+			if !m[strings.ToLower(val)] {
+				return fmt.Errorf("%v must be either 'gpg' or 'internal'", key)
 			}
-			config.BorderCharVertical = char[0]
-		case "border-char-horizontal":
-			char := []rune(val)
-			if len(char) != 1 {
-				return fmt.Errorf("%v must be one and only one character", key)
-			}
-			config.BorderCharHorizontal = char[0]
 		}
 	}
 	return nil
 }
 
-func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
+func LoadConfigFromFile(root *string, accts []string) (*AercConfig, error) {
 	if root == nil {
 		_root := path.Join(xdg.ConfigHome(), "aerc")
 		root = &_root
 	}
-	filename := path.Join(*root, "accounts.conf")
-	if err := checkConfigPerms(filename); err != nil {
-		return nil, err
-	}
-	filename = path.Join(*root, "aerc.conf")
+	filename := path.Join(*root, "aerc.conf")
 
 	// if it doesn't exist copy over the template, then load
 	if _, err := os.Stat(filename); errors.Is(err, os.ErrNotExist) {
+		logging.Debugf("%s not found, installing the system default", filename)
 		if err := installTemplate(*root, "aerc.conf"); err != nil {
 			return nil, err
 		}
 	}
 
+	logging.Infof("Parsing configuration from %s", filename)
+
 	file, err := ini.LoadSources(ini.LoadOptions{
 		KeyValueDelimiters: "=",
 	}, filename)
@@ -557,22 +756,29 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 	file.NameMapper = mapName
 	config := &AercConfig{
 		Bindings: BindingConfig{
-			Global:        NewKeyBindings(),
-			AccountWizard: NewKeyBindings(),
-			Compose:       NewKeyBindings(),
-			ComposeEditor: NewKeyBindings(),
-			ComposeReview: NewKeyBindings(),
-			MessageList:   NewKeyBindings(),
-			MessageView:   NewKeyBindings(),
-			Terminal:      NewKeyBindings(),
+			Global:                 NewKeyBindings(),
+			AccountWizard:          NewKeyBindings(),
+			Compose:                NewKeyBindings(),
+			ComposeEditor:          NewKeyBindings(),
+			ComposeReview:          NewKeyBindings(),
+			MessageList:            NewKeyBindings(),
+			MessageView:            NewKeyBindings(),
+			MessageViewPassthrough: NewKeyBindings(),
+			Terminal:               NewKeyBindings(),
 		},
 
 		ContextualBinds: []BindingConfigContext{},
 
 		Ini: file,
 
+		General: GeneralConfig{
+			PgpProvider:        "internal",
+			UnsafeAccountsConf: false,
+		},
+
 		Ui: UIConfig{
-			IndexFormat:        "%D %-17.17n %s",
+			AutoMarkRead:       true,
+			IndexFormat:        "%-20.20D %-17.17n %Z %s",
 			TimestampFormat:    "2006-01-02 03:04 PM",
 			ThisDayTimeFormat:  "",
 			ThisWeekTimeFormat: "",
@@ -587,10 +793,17 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 			EmptyMessage:        "(no messages)",
 			EmptyDirlist:        "(no folders)",
 			MouseEnabled:        false,
+			ClientThreadsDelay:  50 * time.Millisecond,
 			NewMessageBell:      true,
-			FuzzyFolderComplete: false,
+			FuzzyComplete:       false,
 			Spinner:             "[..]    , [..]   ,  [..]  ,   [..] ,    [..],   [..] ,  [..]  , [..]   ",
 			SpinnerDelimiter:    ",",
+			IconUnencrypted:     "",
+			IconSigned:          "[s]",
+			IconEncrypted:       "[e]",
+			IconSignedEncrypted: "",
+			IconUnknown:         "[s?]",
+			IconInvalid:         "[s!]",
 			DirListFormat:       "%n %>r",
 			DirListDelay:        200 * time.Millisecond,
 			NextMessageOnDelete: true,
@@ -615,6 +828,13 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 				{"Date"},
 				{"Subject"},
 			},
+			ParseHttpLinks: true,
+		},
+
+		Statusline: StatuslineConfig{
+			RenderFormat: "[%a] %S %>%T",
+			Separator:    " | ",
+			DisplayMode:  "",
 		},
 
 		Compose: ComposeConfig{
@@ -631,6 +851,8 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 			QuotedReply:  "quoted_reply",
 			Forwards:     "forward_as_body",
 		},
+
+		Openers: make(map[string][]string),
 	}
 
 	// These bindings are not configurable
@@ -648,43 +870,71 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 		if err := ui.MapTo(&config.General); err != nil {
 			return nil, err
 		}
+		if err := validatePgpProvider(ui); err != nil {
+			return nil, err
+		}
+	}
+
+	logging.Debugf("aerc.conf: [general] %#v", config.General)
+	logging.Debugf("aerc.conf: [ui] %#v", config.Ui)
+	logging.Debugf("aerc.conf: [statusline] %#v", config.Statusline)
+	logging.Debugf("aerc.conf: [viewer] %#v", config.Viewer)
+	logging.Debugf("aerc.conf: [compose] %#v", config.Compose)
+	logging.Debugf("aerc.conf: [filters] %#v", config.Filters)
+	logging.Debugf("aerc.conf: [openers] %#v", config.Openers)
+	logging.Debugf("aerc.conf: [triggers] %#v", config.Triggers)
+	logging.Debugf("aerc.conf: [templates] %#v", config.Templates)
+
+	filename = path.Join(*root, "accounts.conf")
+	if !config.General.UnsafeAccountsConf {
+		if err := checkConfigPerms(filename); err != nil {
+			return nil, err
+		}
 	}
 
 	accountsPath := path.Join(*root, "accounts.conf")
-	if accounts, err := loadAccountConfig(accountsPath); err != nil {
+	logging.Infof("Parsing accounts configuration from %s", accountsPath)
+	if accounts, err := loadAccountConfig(accountsPath, accts); err != nil {
 		return nil, err
 	} else {
 		config.Accounts = accounts
 	}
 
+	for _, acct := range config.Accounts {
+		logging.Debugf("accounts.conf: [%s] from = %s", acct.Name, acct.From)
+	}
+
 	filename = path.Join(*root, "binds.conf")
-	binds, err := ini.Load(filename)
-	if err != nil {
+	if _, err := os.Stat(filename); errors.Is(err, os.ErrNotExist) {
+		logging.Debugf("%s not found, installing the system default", filename)
 		if err := installTemplate(*root, "binds.conf"); err != nil {
 			return nil, err
 		}
-		if binds, err = ini.Load(filename); err != nil {
-			return nil, err
-		}
+	}
+	logging.Infof("Parsing key bindings configuration from %s", filename)
+	binds, err := ini.Load(filename)
+	if err != nil {
+		return nil, err
 	}
 
 	baseGroups := map[string]**KeyBindings{
-		"default":         &config.Bindings.Global,
-		"compose":         &config.Bindings.Compose,
-		"messages":        &config.Bindings.MessageList,
-		"terminal":        &config.Bindings.Terminal,
-		"view":            &config.Bindings.MessageView,
-		"compose::editor": &config.Bindings.ComposeEditor,
-		"compose::review": &config.Bindings.ComposeReview,
+		"default":           &config.Bindings.Global,
+		"compose":           &config.Bindings.Compose,
+		"messages":          &config.Bindings.MessageList,
+		"terminal":          &config.Bindings.Terminal,
+		"view":              &config.Bindings.MessageView,
+		"view::passthrough": &config.Bindings.MessageViewPassthrough,
+		"compose::editor":   &config.Bindings.ComposeEditor,
+		"compose::review":   &config.Bindings.ComposeReview,
 	}
 
 	// Base Bindings
 	for _, sectionName := range binds.SectionStrings() {
 		// Handle :: delimeter
-		baseSectionName := strings.Replace(sectionName, "::", "////", -1)
+		baseSectionName := strings.ReplaceAll(sectionName, "::", "////")
 		sections := strings.Split(baseSectionName, ":")
 		baseOnly := len(sections) == 1
-		baseSectionName = strings.Replace(sections[0], "////", "::", -1)
+		baseSectionName = strings.ReplaceAll(sections[0], "////", "::")
 
 		group, ok := baseGroups[strings.ToLower(baseSectionName)]
 		if !ok {
@@ -692,7 +942,7 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 		}
 
 		if baseOnly {
-			err = config.LoadBinds(binds, baseSectionName, group, logger)
+			err = config.LoadBinds(binds, baseSectionName, group)
 			if err != nil {
 				return nil, err
 			}
@@ -705,6 +955,7 @@ func LoadConfigFromFile(root *string, logger *log.Logger) (*AercConfig, error) {
 			contextBind.Bindings.Globals = false
 		}
 	}
+	logging.Debugf("binds.conf: %#v", config.Bindings)
 
 	return config, nil
 }
@@ -742,8 +993,7 @@ func LoadBindingSection(sec *ini.Section) (*KeyBindings, error) {
 	return bindings, nil
 }
 
-func (config *AercConfig) LoadBinds(binds *ini.File, baseName string, baseGroup **KeyBindings, logger *log.Logger) error {
-
+func (config *AercConfig) LoadBinds(binds *ini.File, baseName string, baseGroup **KeyBindings) error {
 	if sec, err := binds.GetSection(baseName); err == nil {
 		binds, err := LoadBindingSection(sec)
 		if err != nil {
@@ -768,11 +1018,10 @@ func (config *AercConfig) LoadBinds(binds *ini.File, baseName string, baseGroup
 			return err
 		}
 
-		contextualBind :=
-			BindingConfigContext{
-				Bindings:    binds,
-				BindContext: baseName,
-			}
+		contextualBind := BindingConfigContext{
+			Bindings:    binds,
+			BindContext: baseName,
+		}
 
 		var index int
 		if strings.Contains(sectionName, "=") {
@@ -797,10 +1046,14 @@ func (config *AercConfig) LoadBinds(binds *ini.File, baseName string, baseGroup
 				}
 			}
 			if !valid {
-				logger.Printf("Tried to define binds for unexistent account: %v\n", acctName)
+				logging.Warnf("binds.conf: unexistent account: %s", acctName)
 				continue
 			}
 			contextualBind.ContextType = BIND_CONTEXT_ACCOUNT
+		case "folder":
+			// No validation needed. If the folder doesn't exist, the binds
+			// never get used
+			contextualBind.ContextType = BIND_CONTEXT_FOLDER
 		default:
 			return fmt.Errorf("Unknown Context Bind Section: %s", sectionName)
 		}
@@ -814,12 +1067,16 @@ func (config *AercConfig) LoadBinds(binds *ini.File, baseName string, baseGroup
 // printing the fix on stdout and returning an error
 func checkConfigPerms(filename string) error {
 	info, err := os.Stat(filename)
-	if err != nil {
+	if errors.Is(err, os.ErrNotExist) {
 		return nil // disregard absent files
 	}
+	if err != nil {
+		return err
+	}
+
 	perms := info.Mode().Perm()
 	// group or others have read access
-	if perms&044 != 0 {
+	if perms&0o44 != 0 {
 		fmt.Fprintf(os.Stderr, "The file %v has too open permissions.\n", filename)
 		fmt.Fprintln(os.Stderr, "This is a security issue (it contains passwords).")
 		fmt.Fprintf(os.Stderr, "To fix it, run `chmod 600 %v`\n", filename)
@@ -841,14 +1098,15 @@ func (ui *UIConfig) loadStyleSet(styleSetDirs []string) error {
 	ui.style = NewStyleSet()
 	err := ui.style.LoadStyleSet(ui.StyleSetName, styleSetDirs)
 	if err != nil {
-		return fmt.Errorf("Unable to load default styleset: %s", err)
+		return fmt.Errorf("Unable to load default styleset: %w", err)
 	}
 
 	return nil
 }
 
 func (config AercConfig) mergeContextualUi(baseUi UIConfig,
-	contextType ContextType, s string) UIConfig {
+	contextType ContextType, s string,
+) UIConfig {
 	for _, contextualUi := range config.ContextualUis {
 		if contextualUi.ContextType != contextType {
 			continue
@@ -858,7 +1116,10 @@ func (config AercConfig) mergeContextualUi(baseUi UIConfig,
 			continue
 		}
 
-		mergo.Merge(&baseUi, contextualUi.UiConfig, mergo.WithOverride)
+		err := mergo.Merge(&baseUi, contextualUi.UiConfig, mergo.WithOverride)
+		if err != nil {
+			logging.Warnf("merge ui failed: %v", err)
+		}
 		if contextualUi.UiConfig.StyleSetName != "" {
 			baseUi.style = contextualUi.UiConfig.style
 		}
@@ -868,14 +1129,18 @@ func (config AercConfig) mergeContextualUi(baseUi UIConfig,
 	return baseUi
 }
 
-func (config AercConfig) GetUiConfig(params map[ContextType]string) UIConfig {
+func (config AercConfig) GetUiConfig(params map[ContextType]string) *UIConfig {
 	baseUi := config.Ui
 
 	for k, v := range params {
 		baseUi = config.mergeContextualUi(baseUi, k, v)
 	}
 
-	return baseUi
+	return &baseUi
+}
+
+func (config *AercConfig) GetContextualUIConfigs() []UIConfigContext {
+	return config.ContextualUis
 }
 
 func (uiConfig UIConfig) GetStyle(so StyleObject) tcell.Style {
@@ -887,10 +1152,20 @@ func (uiConfig UIConfig) GetStyleSelected(so StyleObject) tcell.Style {
 }
 
 func (uiConfig UIConfig) GetComposedStyle(base StyleObject,
-	styles []StyleObject) tcell.Style {
+	styles []StyleObject,
+) tcell.Style {
 	return uiConfig.style.Compose(base, styles)
 }
 
 func (uiConfig UIConfig) GetComposedStyleSelected(base StyleObject, styles []StyleObject) tcell.Style {
 	return uiConfig.style.ComposeSelected(base, styles)
 }
+
+func contains(list []string, v string) bool {
+	for _, item := range list {
+		if item == v {
+			return true
+		}
+	}
+	return false
+}
