@@ -10,6 +10,7 @@ import (
 	"path"
 	"strconv"
 	"strings"
+	"sync"
 
 	"github.com/gdamore/tcell/v2"
 	"github.com/go-ini/ini"
@@ -39,14 +40,12 @@ const (
 )
 
 type AccountWizard struct {
-	ui.Invalidatable
 	aerc      *Aerc
 	conf      *config.AercConfig
 	step      int
 	steps     []*ui.Grid
 	focus     int
 	temporary bool
-	testing   bool
 	// CONFIGURE_BASICS
 	accountName *ui.TextInput
 	email       *ui.TextInput
@@ -73,6 +72,21 @@ type AccountWizard struct {
 	complete []ui.Interactive
 }
 
+func showPasswordWarning(aerc *Aerc) {
+	title := "The Wizard will store your passwords in plaintext"
+	text := "It is recommended to remove the plaintext passwords " +
+		"and use your personal password store with " +
+		"'source-cred-cmd' and 'outgoing-cred-cmd' after the setup."
+	warning := NewSelectorDialog(
+		title, text, []string{"OK"}, 0,
+		aerc.SelectedAccountUiConfig(),
+		func(_ string, _ error) {
+			aerc.CloseDialog()
+		},
+	)
+	aerc.AddDialog(warning)
+}
+
 func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 	wizard := &AccountWizard{
 		accountName:  ui.NewTextInput("", &conf.Ui).Prompt("> "),
@@ -120,10 +134,12 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		wizard.imapUri()
 		wizard.smtpUri()
 	})
+	var once sync.Once
 	wizard.imapPassword.OnChange(func(_ *ui.TextInput) {
 		wizard.smtpPassword.Set(wizard.imapPassword.String())
 		wizard.imapUri()
 		wizard.smtpUri()
+		once.Do(func() { showPasswordWarning(aerc) })
 	})
 	wizard.smtpServer.OnChange(func(_ *ui.TextInput) {
 		wizard.smtpUri()
@@ -194,7 +210,7 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 					}
 					wizard.imapUri()
 				}
-				hostport, srv = getSRV(server, []string{"submission"})
+				hostport, _ = getSRV(server, []string{"submission"})
 				if hostport != "" {
 					wizard.smtpServer.Set(hostport)
 					wizard.smtpMode = SMTP_STARTTLS
@@ -207,9 +223,6 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 	wizard.basics = []ui.Interactive{
 		wizard.accountName, wizard.fullName, wizard.email, selector,
 	}
-	basics.OnInvalidate(func(_ ui.Drawable) {
-		wizard.Invalidate()
-	})
 
 	incoming := ui.NewGrid().Rows([]ui.GridSpec{
 		{Strategy: ui.SIZE_EXACT, Size: ui.Const(3)}, // Introduction
@@ -286,9 +299,6 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		wizard.imapUsername, wizard.imapPassword, wizard.imapServer,
 		imapMode, selector,
 	}
-	incoming.OnInvalidate(func(_ ui.Drawable) {
-		wizard.Invalidate()
-	})
 
 	outgoing := ui.NewGrid().Rows([]ui.GridSpec{
 		{Strategy: ui.SIZE_EXACT, Size: ui.Const(3)}, // Introduction
@@ -382,9 +392,6 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 		wizard.smtpUsername, wizard.smtpPassword, wizard.smtpServer,
 		smtpMode, copySent, selector,
 	}
-	outgoing.OnInvalidate(func(_ ui.Drawable) {
-		wizard.Invalidate()
-	})
 
 	complete := ui.NewGrid().Rows([]ui.GridSpec{
 		{Strategy: ui.SIZE_EXACT, Size: ui.Const(7)},  // Introduction
@@ -414,9 +421,6 @@ func NewAccountWizard(conf *config.AercConfig, aerc *Aerc) *AccountWizard {
 	})
 	complete.AddChild(selector).At(1, 0)
 	wizard.complete = []ui.Interactive{selector}
-	complete.OnInvalidate(func(_ ui.Drawable) {
-		wizard.Invalidate()
-	})
 
 	wizard.steps = []*ui.Grid{basics, incoming, outgoing, complete}
 	return wizard
@@ -493,20 +497,21 @@ func (wizard *AccountWizard) finish(tutorial bool) {
 		return
 	}
 	sec, _ = file.NewSection(wizard.accountName.String())
-	sec.NewKey("source", wizard.imapUrl.String())
-	sec.NewKey("outgoing", wizard.smtpUrl.String())
-	sec.NewKey("default", "INBOX")
+	// these can't really fail
+	sec.NewKey("source", wizard.imapUrl.String())   //nolint:errcheck // can't fail. option shadowing is not enabled and the key is not empty
+	sec.NewKey("outgoing", wizard.smtpUrl.String()) //nolint:errcheck // can't fail. option shadowing is not enabled and the key is not empty
+	sec.NewKey("default", "INBOX")                  //nolint:errcheck // can't fail. option shadowing is not enabled and the key is not empty
 	if wizard.smtpMode == SMTP_STARTTLS {
-		sec.NewKey("smtp-starttls", "yes")
+		sec.NewKey("smtp-starttls", "yes") //nolint:errcheck // can't fail. option shadowing is not enabled and the key is not empty
 	}
-	sec.NewKey("from", fmt.Sprintf("%s <%s>",
+	sec.NewKey("from", fmt.Sprintf("%s <%s>", //nolint:errcheck // can't fail. option shadowing is not enabled and the key is not empty
 		wizard.fullName.String(), wizard.email.String()))
 	if wizard.copySent {
-		sec.NewKey("copy-to", "Sent")
+		sec.NewKey("copy-to", "Sent") //nolint:errcheck // can't fail. option shadowing is not enabled and the key is not empty
 	}
 
 	if !wizard.temporary {
-		f, err := os.OpenFile(accountsConf, os.O_WRONLY|os.O_CREATE, 0600)
+		f, err := os.OpenFile(accountsConf, os.O_WRONLY|os.O_CREATE, 0o600)
 		if err != nil {
 			wizard.errorFor(nil, err)
 			return
@@ -522,7 +527,7 @@ func (wizard *AccountWizard) finish(tutorial bool) {
 		Default:  "INBOX",
 		From:     sec.Key("from").String(),
 		Source:   sec.Key("source").String(),
-		Outgoing: sec.Key("outgoing").String(),
+		Outgoing: config.RemoteConfig{Value: sec.Key("outgoing").String()},
 	}
 	if wizard.smtpMode == SMTP_STARTTLS {
 		account.Params = map[string]string{
@@ -534,8 +539,7 @@ func (wizard *AccountWizard) finish(tutorial bool) {
 	}
 	wizard.conf.Accounts = append(wizard.conf.Accounts, account)
 
-	view, err := NewAccountView(wizard.aerc, wizard.conf, &account,
-		wizard.aerc.logger, wizard.aerc, nil)
+	view, err := NewAccountView(wizard.aerc, wizard.conf, &account, wizard.aerc, nil)
 	if err != nil {
 		wizard.aerc.NewTab(errorScreen(err.Error(), wizard.conf.Ui),
 			account.Name)
@@ -648,7 +652,7 @@ func (wizard *AccountWizard) smtpUri() url.URL {
 }
 
 func (wizard *AccountWizard) Invalidate() {
-	wizard.DoInvalidate(wizard)
+	ui.Invalidate()
 }
 
 func (wizard *AccountWizard) Draw(ctx *ui.Context) {
@@ -690,8 +694,7 @@ func (wizard *AccountWizard) Focus(focus bool) {
 
 func (wizard *AccountWizard) Event(event tcell.Event) bool {
 	interactive := wizard.getInteractive()
-	switch event := event.(type) {
-	case *tcell.EventKey:
+	if event, ok := event.(*tcell.EventKey); ok {
 		switch event.Key() {
 		case tcell.KeyUp:
 			fallthrough
