@@ -50,10 +50,7 @@ func (imapw *IMAPWorker) handleFetchMessageHeaders(
 			if err != nil {
 				logging.Errorf(
 					"message %d: could not read header: %v", _msg.Uid, err)
-				imapw.worker.PostMessage(&types.Error{
-					Message: types.RespondTo(msg),
-					Error:   err,
-				}, nil)
+				imapw.worker.PostMessageInfoError(msg, _msg.Uid, err)
 				return nil
 			}
 			header := &mail.Header{Header: message.Header{Header: textprotoHeader}}
@@ -97,6 +94,7 @@ func (imapw *IMAPWorker) handleFetchMessageBodyPart(
 		partBodySection.Specifier = imap.TextSpecifier
 	}
 	partBodySection.Path = msg.Part
+	partBodySection.Peek = true
 
 	items := []imap.FetchItem{
 		imap.FetchEnvelope,
@@ -123,7 +121,10 @@ func (imapw *IMAPWorker) handleFetchMessageBodyPart(
 
 			part, err := message.New(message.Header{Header: h},
 				_msg.GetBody(&partBodySection))
-			if err != nil {
+			if message.IsUnknownCharset(err) {
+				logging.Warnf("unknown charset encountered "+
+					"for uid %d", _msg.Uid)
+			} else if err != nil {
 				return fmt.Errorf("failed to create message reader: %w", err)
 			}
 
@@ -150,7 +151,9 @@ func (imapw *IMAPWorker) handleFetchFullMessages(
 	msg *types.FetchFullMessages,
 ) {
 	logging.Infof("Fetching full messages: %v", msg.Uids)
-	section := &imap.BodySectionName{}
+	section := &imap.BodySectionName{
+		Peek: true,
+	}
 	items := []imap.FetchItem{
 		imap.FetchEnvelope,
 		imap.FetchFlags,
