@@ -5,7 +5,6 @@ import (
 	"io"
 
 	"github.com/emersion/go-maildir"
-	"github.com/emersion/go-message"
 
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/lib"
@@ -46,7 +45,7 @@ func (m Message) SetFlags(flags []maildir.Flag) error {
 func (m Message) SetOneFlag(flag maildir.Flag, enable bool) error {
 	flags, err := m.Flags()
 	if err != nil {
-		return fmt.Errorf("could not read previous flags: %v", err)
+		return fmt.Errorf("could not read previous flags: %w", err)
 	}
 	if enable {
 		flags = append(flags, flag)
@@ -85,9 +84,9 @@ func (m Message) NewBodyPartReader(requestedParts []int) (io.Reader, error) {
 		return nil, err
 	}
 	defer f.Close()
-	msg, err := message.Read(f)
+	msg, err := lib.ReadMessage(f)
 	if err != nil {
-		return nil, fmt.Errorf("could not read message: %v", err)
+		return nil, fmt.Errorf("could not read message: %w", err)
 	}
 	return lib.FetchEntityPartReader(msg, requestedParts)
 }
